{ config, pkgs, lib, agenix, ... }:

{

    imports = [

		agenix.nixosModule

		../../modules

		# Reverse proxy
		./services/nginx.nix

		# Website
		./services/website.nix

    ];

	networking.hostName = "sif";

	boot.loader = {
		grub.enable = false;
		generic-extlinux-compatible.enable = true;
	};

	documentation.enable = false;

	environment.systemPackages = with pkgs; [
		git
		vim
	];

    age.secrets."website".file = ../../secrets/website.age;

	system.stateVersion = "21.11";

	# Custom modules defined by myself.
	custom = {

    	gui.enable = false;

		kak = {
			enable  = true;
			ide.nix = true;
		};

		net.interfaces = [
    		"eth0"
		];
		
	};


	users.users."riley" = {
		isNormalUser = true;
		extraGroups = [ "wheel" ];
	};

	networking.interfaces.eth0.useDHCP = true;

	boot.kernelPackages = pkgs.linuxPackages_latest;

	fileSystems."/" = {
		device = "/dev/disk/by-label/NIXOS_SD";
		fsType = "ext4";
	};

	nix = {
		distributedBuilds = true;
		buildMachines = [{

            hostName = "thor";
            sshUser  = "riley";
            sshKey   = "/root/.ssh/id_ed25519";

            systems = [
                "x86_64-linux"
                "aarch64-linux"
            ];

            supportedFeatures = [
                "nixos-test"
                "benchmark"
                "big-parallel"
                "kvm"
            ];

		}];
	};

}
