{ config, lib, pkgs, agenix, ... }:

{
    imports = [
        agenix.nixosModule
        ./hardware-configuration.nix
        ../../modules
    ];

    custom = {
        gui = {
            enable = true;
            theme = (import ../../colors.nix).dark;
        };
        kak = {

            enable = true;

            haskell = true;
            rust = true;
            nix = true;

        };
    };

    system.stateVersion = "21.11";

    networking = {
        hostName = "thor";
        interfaces.enp9s0.useDHCP = true;
    };

    boot = {
        loader.systemd-boot = {
            enable = true;
            editor = false;
            configurationLimit = 10;
        };

        loader.efi = {
            canTouchEfiVariables = true;
        };
    };
    home-manager.users.riley.home.stateVersion = "21.11";

    users.users."riley".packages = with pkgs; [
        minecraft
        cockatrice
    ];

    programs.steam = {
        enable = true;
    };

    hardware.opengl = {
        enable = true;
        driSupport = true;
        driSupport32Bit = true;
    };

    services.xserver.videoDrivers = [ "nvidia" ];

    age.secrets."website-secret".file = ../../secrets/website.age;

    devices = {

        # Audio devices
        audio = {

            # Outputs
            speakers = "alsa_output.pci-0000_0c_00.4.analog-stereo";
            external = "alsa_output.pci-0000_0a_00.1.hdmi-stereo";
            headset  = null; # Thor doesn't have bluetooth support (yet)

            # Inputs
            main-mic = "alsa_input.usb-046d_C922_Pro_Stream_Webcam_CC9E75BF-02.analog-stereo";
            
        };

        # Displays
        video.displays = {

            # Connected to the ultrawide output
            "HDMI-1" = {
                primary = true;
                position = [ 0 0 ];
            };

            # Vertical output to the right of HDMI-1
            "DP-2" = {
                rotate = "left";
                position = [ 2560 ((1080 - 1920) / 2) ];
            };
            
        };

    };

}
